import maya.cmds as cmds
from functools import partial


#############################################################
# Segment Scale Compensateが有効のジョイント
#############################################################
def getJointsWithSegmentScaleCompensateOn():
    '''
    （ヘルパー関数）
    Segment Scale Compensate が有効になっているジョイントのリストを返します。
    '''
    joints = cmds.ls(type='joint')
    invalidJoints = []
    for joint in joints:
        v = cmds.getAttr(f'{joint}.segmentScaleCompensate')
        if v:
            invalidJoints.append(joint)
    return invalidJoints
    

def checkJointSegmentScaleCompensate(autoFix):
    '''
    （ボタンから呼び出す関数）
    Segment Scale Compensate のチェックと修正
    '''
    print('Check Joint Scale Compensate is OFF.')
    invalidJoints = []

    if autoFix:
        # 自動修正なら、値をチェックする前に直しておきます。
        invalidJoints = getJointsWithSegmentScaleCompensateOn()
        for joint in invalidJoints:
            print(f'{joint}のsegmentScaleCompensateを無効にします。')
            cmds.setAttr(f'{joint}.segmentScaleCompensate', False)

    # 改めて問題のあるノードを確認。
    invalidJoints = getJointsWithSegmentScaleCompensateOn()
    if len(invalidJoints):
        cmds.warning(f'ジョイントのsegmentScaleCompensate が有効のままです！{invalidJoints}')
    
    return invalidJoints
        


#############################################################
# 回転値が0.0になっていないJoint
#############################################################
def getJointsWithRotated():
    '''
    （ヘルパー関数）
    回転値が0.0以外になっているジョイントのリストを返します。
    '''
    joints = cmds.ls(type='joint')
    invalidJoints = []
    for joint in joints:
        v = cmds.getAttr(f'{joint}.rotate')[0]
        if v[0] != 0.0 or v[1] != 0.0 or v[2] != 0.0:
            invalidJoints.append(joint)
    return invalidJoints


def checkJointRotation(autoFix):
    '''
    （ボタンから呼び出す関数）
    ジョイントの回転値が0.0になっているかのチェックと修正
    '''
    print('Check Joint Rotation is Zero.')
    invalidJoints = []

    if autoFix:
        # 自動修正なら、値をチェックする前に直しておきます。
        invalidJoints = getJointsWithRotated()
        for joint in invalidJoints:
            print(f'{joint}の回転値を0.0にします。')
            cmds.setAttr(f'{joint}.rotate', 0.0, 0.0, 0.0, type='double3')
    
    # 改めて問題のあるノードを確認。
    invalidJoints = getJointsWithRotated()
    if len(invalidJoints):
        cmds.warning(f'ジョイントの回転値に0.0以外の値が入っています！{invalidJoints}')
    
    return invalidJoints


#############################################################
# ネームスペースがある
#############################################################
def getValidNamespace():
    '''
    （ヘルパー関数）
    ネームスペースをリストとして返します。
    '''
    # Name Space EditorであるnamespaceTreeView.melを参考に。
    namespaces = cmds.namespaceInfo(':', recurse=True, listOnlyNamespaces=True, absoluteName=True)

    # internal namespaceは取り除きます。
    namespaces.remove(':UI')
    namespaces.remove(':shared')

    return namespaces

def checkNamespace(autoFix):
    '''
    （ボタンから呼び出す関数）
    ネームスペースがあるかどうかチェックします。
    修正ではネームスペースを削除して、全てのノードをルートネームスペースにします。
    （ノード名の衝突を回避するためにノード名が変わる可能性があります。）
    '''
    print('Check Namespace.')
    namespaces = []

    # ネームスペースは親子構造になるため、逆順で子どもの方から削除すれば多分大丈夫・・・。
    if autoFix:
        namespaces = getValidNamespace()
        namespaces.reverse()
        for ns in namespaces:
            print(f'ネームスペースを削除します。 {ns}')
            cmds.namespace(removeNamespace=ns, mergeNamespaceWithRoot=True)

    # 自動修正があった場合も含め、もう一度チェックします。
    namespaces = getValidNamespace()
    if len(namespaces):
        cmds.warning(f'シーンにネームスペースが残っています。{namespaces}')
    
    return namespaces


#############################################################
# 使用していないエクスプレッションがある
#############################################################
def getUnusedExpressions():
    '''
    （ヘルパー関数）
    使用していないExpressionノードをリストとして返します。
    使用していない＝Expressionから下流への接続先が無いもの、とします。
    '''
    unusedExpressions = []
    expressions = cmds.ls(type='expression')
    for exp in expressions:
        connections = cmds.listConnections(exp, s=False, d=True)
        if not connections:
            unusedExpressions.append(exp)
    return unusedExpressions


def checkExpression(*autoFix):
    '''
    （ボタンから呼び出す関数）
    使用していないExpressionノードのチェックと修正。
    '''
    print('Check Unused Expression.')
    unusedExpressions = []

    # 使っていないエクスプレッションとは、出力の接続がないものとします。
    if autoFix:
        unusedExpressions = getUnusedExpressions()
        if len(unusedExpressions):
            print(f'使用していないExpressionノードがあるので、削除します。 {unusedExpressions}')
            cmds.delete(unusedExpressions)

    # 自動修正があった場合も含め、もう一度チェックします。
    unusedExpressions = getUnusedExpressions()
    if len(unusedExpressions):
        cmds.warning(f'シーンに使用していないExpressionノードが残っています。{unusedExpressions}')
    
    return unusedExpressions



#############################################################
# GUI部分
#############################################################

def setWidgetColor(widget, isValid):
    '''
    isvalidの設定にもとづいて、ボタンの色を変更します。
    '''
    color = (0.7, 0.9, 0.1) # とりあえずOKの色で初期化
    if not isValid:
        color = (0.9, 0.1, 0.1) # NGの場合の色
    cmds.button(widget, e=True, bgc=color)

def runChecker(*args):
    '''
    （ボタンから呼び出す関数）
    引数にもとづき、各チェック用関数を実行します。
    argsには次が含まれている必要があります。[チェック用関数], [ボタンのUI名], [自動修正有無のチェックボックスのUI名]
    '''
    function = args[0]
    button = args[1]
    checkBox = args[2]

    # 自動修正の有無を確認の上、チェック用関数を実行します。
    autoFix = cmds.checkBox(checkBox, q=True, v=True)
    invalids = function(autoFix)

    # 何かしら無効なものが返ってきていればNG扱いで、ボタンの色を変更します。
    isValid = len(invalids) == 0
    setWidgetColor(args[1], isValid)

    # invalidsの中身のノード名が、選択可能なら選択。
    selectable = []
    for invalid in invalids:
        if cmds.objExists(invalid):
            selectable.append(invalid)
    
    if len(selectable):
        cmds.select(selectable)
    

def runCheckerAll(*args):
    '''
    （ボタンから呼び出す関数）
    全てのチェック項目を実行する場合はこちら。
    argsにはボタンに渡したコマンドが入っているので、順に実行します。
    '''
    for func in args[0]:
        func()


def characterChecker():
    '''
    （メイン関数）
    ウインドウを作ります。
    '''

    # はじめにテスト項目を登録します。
    # あとでこの情報に従ってボタンなどを作ります。
    # テストを追加するときはここに項目を追加して、実行するチェック用関数も追加します。
    tests = [] # ラベル、チェックで実行したいコード（関数）の配列とします。
    tests.append( ['Joint Scale Compensateがオフになっているか？', checkJointSegmentScaleCompensate] )
    tests.append( ['Joint Rotation がゼロか？', checkJointRotation] )
    tests.append( ['Namespaceがない状態か？', checkNamespace] )
    tests.append( ['使用していないExpressionが残っていないか？', checkExpression])


    # ウインドウを作ります。
    cmds.window(t='Character Checker', w=500, h=300)

    # 上から下にレイアウトするためにcolumnLayoutにします。
    cmds.columnLayout()

    #　ツールの説明。とりあえず・・・。
    cmds.text('　　ボタンを実行すると、各項目のチェックを行います。\n')
    cmds.text('　　実行ボタン　　　自動修正　　　　概要')

    buttonFunctions = []
    for t in tests:
        # rowLayoutで横並びでGUIを作っていきます。左に置くボタンは隙間を作りたいのでcolumnAttachで少し左からオフセットさせます
        cmds.rowLayout(nc=3, cw3=(100, 50, 300), columnAttach=(1, 'left', 20))

        # とりあえず左からの順番でGUIを作成。
        btn = cmds.button(l='Run', w=60)
        cb = cmds.checkBox(l='', v=False)
        cmds.text(l=t[0])
        cmds.setParent('..')

        # 見やすくするために上下に隙間を作ります。
        cmds.separator(h=10) 

        # 改めて、ボタンのコマンドを設定します。
        # 関数にテキストのGUI名をわたして、色替えなどに使わせます。
        # functoolsモジュールからインポートしておいたpartialをコマンドに使います。(実行したい関数, 引数)を指定します。
        cmd = partial(runChecker, t[1], btn, cb)
        cmds.button(btn, e=True, c=cmd)
        buttonFunctions.append(cmd)

    # 全て実行するボタン。
    cmds.rowLayout(nc=1, cw1=200, columnAttach=(1, 'left', 20))
    cmds.button(l='Run All', w=120, c=partial(runCheckerAll, buttonFunctions))
    cmds.setParent('..')

    cmds.showWindow()

characterChecker()